using System;
using System.Collections.Generic;
using System.Collections;
using SdlDotNet;

namespace Physics.Config.Files
{
    class Effects_cfg : ConfigFile
    {
        private String readFile = "Effects.cfg";

        public InputSet gravitykey = new InputSet(Key.G);
        public InputSet drawVelocitykey = new InputSet(Key.V);

        public InputSet mouseForcekey = new InputSet(MouseButton.PrimaryButton);
        public InputSet mouseWindkey = new InputSet(MouseButton.SecondaryButton);
        public InputSet gravWellkey = new InputSet(MouseButton.MiddleButton);
        public InputSet mouseLockkey = new InputSet(Key.Return); // should accept Enter for this

        public InputSet airFrictionkey = new InputSet(Key.U);
        public InputSet antiAliasingkey = new InputSet(Key.L);

        private Hashtable map;

        public Effects_cfg()
        {
            List<ConfigEntry> lst = Read(configFolder + readFile);

            CreateStringToKeyMap();

            foreach (ConfigEntry cfge in lst)
            {
                if (map.ContainsKey(cfge.Varible))
                    ((InputSet)map[cfge.Varible]).SetTo(cfge.Keys);
            }
        }

        public InputSet gravity
        { get { return gravitykey; } }
        public InputSet drawVelocity
        { get { return drawVelocitykey; } }

        public InputSet mouseForce
        { get { return mouseForcekey; } }
        public InputSet mouseWind
        { get { return mouseWindkey; } }
        public InputSet gravWell
        { get { return gravWellkey; } }
        public InputSet mouseLock
        { get { return mouseLockkey; } }

        public InputSet airFriction
        { get { return airFrictionkey; } }
        public InputSet antiAliasing
        { get { return antiAliasingkey; } }

        private void CreateStringToKeyMap()
        {
            if (map != null)
                return;

            map = new Hashtable();

            map.Add("gravity", gravitykey);
            map.Add("drawVelocity", drawVelocitykey);

            map.Add("mouseForce", mouseForcekey);
            map.Add("mouseWind", mouseWindkey);
            map.Add("gravWell", gravWellkey);
            map.Add("mouseLock", mouseLockkey);

            map.Add("airFriction", airFrictionkey);
            map.Add("antiAliasing", antiAliasingkey);
        }
    }
}
