using System;
using SdlDotNet;
using System.Collections;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace Physics.Config
{
    class InputSet
    {
        private List<Key> keyList;
        private List<MouseButton> butList;

        private static Hashtable keyMap;
        private static Hashtable butMap;

        public InputSet(Key key)
        {
            keyList = new List<Key>();
            butList = new List<MouseButton>();
            keyList.Add(key);
        }
        public InputSet(MouseButton mouseButton)
        {
            keyList = new List<Key>();
            butList = new List<MouseButton>();
            butList.Add(mouseButton);
        }

        public void SetTo(String txt)
        {
            CreateStringToKeyMap();
            CreateStringToButMap();

            keyList = new List<Key>();
            butList = new List<MouseButton>();

            Regex plusSplit = new Regex("\\+");

            String[] a = plusSplit.Split(txt);

            foreach (String s in a)
            {
                string s1 = s.Trim();

                if (keyMap.ContainsKey(s1))
                    keyList.Add((Key)keyMap[s1]);
                if (butMap.ContainsKey(s1))
                    butList.Add((MouseButton)butMap[s1]);
            }
        }

        public bool wasPressed()
        {
            int total = keyList.Count + butList.Count;
            int isCount = 0;
            int wasCount = 0;

            foreach (Key key in keyList)
            {
                if (InputMonitor.isPressed(key))
                    isCount++;
                if (InputMonitor.wasPressed(key))
                    wasCount++;
            }
            foreach (MouseButton but in butList)
            {
                if (InputMonitor.isPressed(but))
                    isCount++;
                if (InputMonitor.wasPressed(but))
                    wasCount++;
            }

            return isCount == total && wasCount > 0;
        }
        public bool isPressed()
        {
            int total = keyList.Count + butList.Count;
            int count = 0;

            foreach (Key key in keyList)
            {
                if (InputMonitor.isPressed(key))
                    count++;
            }
            foreach (MouseButton but in butList)
            {
                if (InputMonitor.isPressed(but))
                    count++;
            }

            return count == total;
        }
        public bool wasReleased()
        {
            int total = keyList.Count + butList.Count;
            int isCount = 0;
            int wasCount = 0;

            foreach (Key key in keyList)
            {
                if (InputMonitor.isPressed(key))
                    isCount++;
                if (InputMonitor.wasReleased(key))
                    wasCount++;
            }
            foreach (MouseButton but in butList)
            {
                if (InputMonitor.isPressed(but))
                    isCount++;
                if (InputMonitor.wasReleased(but))
                    wasCount++;
            }

            return isCount + wasCount == total && wasCount > 0;
        }
        public bool isReleased()
        {
            int total = keyList.Count + butList.Count;
            int count = 0;

            foreach (Key key in keyList)
            {
                if (InputMonitor.isReleased(key))
                    count++;
            }
            foreach (MouseButton but in butList)
            {
                if (InputMonitor.isReleased(but))
                    count++;
            }

            return count == total;
        }

        private static void CreateStringToKeyMap()
        {
            if (keyMap != null)
                return;

            keyMap = new Hashtable(100);

            // alt, ctrl, shift
            keyMap.Add("LCtrl", Key.LeftControl);
            keyMap.Add("LeftControl", Key.LeftControl);
            keyMap.Add("LAlt", Key.LeftAlt);
            keyMap.Add("LeftAlt", Key.LeftAlt);
            keyMap.Add("LShift", Key.LeftShift);
            keyMap.Add("LeftShift", Key.LeftShift);
            keyMap.Add("LWin", Key.LeftWindows);
            keyMap.Add("LeftWindows", Key.LeftWindows);

            keyMap.Add("RCtrl", Key.RightControl);
            keyMap.Add("RightControl", Key.RightControl);
            keyMap.Add("RAlt", Key.RightAlt);
            keyMap.Add("RightAlt", Key.RightAlt);
            keyMap.Add("RShift", Key.RightShift);
            keyMap.Add("RightShift", Key.RightShift);
            keyMap.Add("RWin", Key.RightWindows);
            keyMap.Add("RightWindows", Key.RightWindows);

            keyMap.Add("Esc", Key.Escape);
            keyMap.Add("Escape", Key.Escape);

            keyMap.Add("Return", Key.Return);
            keyMap.Add("Enter", Key.Return);

            keyMap.Add("Insert", Key.Insert);
            keyMap.Add("Home", Key.Home);
            keyMap.Add("Delete", Key.Delete);
            keyMap.Add("End", Key.End);
            keyMap.Add("PageUp", Key.PageUp);
            keyMap.Add("PageDown", Key.PageDown);

            keyMap.Add("Minus", Key.Minus);
            keyMap.Add("Equal", Key.Equals);
            keyMap.Add("Equals", Key.Equals);
            keyMap.Add("LeftBracket", Key.LeftBracket);
            keyMap.Add("LBracket", Key.LeftBracket);
            keyMap.Add("RightBracket", Key.RightBracket);
            keyMap.Add("RBracket", Key.RightBracket);
            keyMap.Add("Backslash", Key.Backslash);
            keyMap.Add("Slash", Key.Slash);
            keyMap.Add("Semicolon", Key.Semicolon);
            keyMap.Add("Semi", Key.Semicolon);
            keyMap.Add("Quote", Key.Quote);
            keyMap.Add("Comma", Key.Comma);
            keyMap.Add("Period", Key.Period);
            keyMap.Add("Space", Key.Space);
            keyMap.Add("BSpace", Key.Backspace);
            keyMap.Add("Backspace", Key.Backspace);
            keyMap.Add("BackSpace", Key.Backspace);
            keyMap.Add("RightMeta", Key.RightMeta);
            keyMap.Add("RMeta", Key.RightMeta);

            keyMap.Add("Tab", Key.Tab);
            keyMap.Add("BackQuote", Key.BackQuote);
            keyMap.Add("BQuote", Key.BackQuote);
            keyMap.Add("CapsLock", Key.CapsLock);
            keyMap.Add("Caps", Key.CapsLock);

            keyMap.Add("UpArrow", Key.UpArrow);
            keyMap.Add("DownArrow", Key.DownArrow);
            keyMap.Add("LeftArrow", Key.LeftArrow);
            keyMap.Add("RightArrow", Key.RightArrow);

            // add all the capital and lowercase letters
            for (int i = (int)Key.A; i <= (int)Key.Z; i++)
            {
                keyMap.Add("" + (char)i, (Key)i);           // uppercase
                keyMap.Add("" + (char)(i - 32), (Key)i);    // lowercase
            }

            // add all the numbers
            for (int i = (int)Key.Zero; i <= (int)Key.Nine; i++)
            {
                keyMap.Add("" + (char)i, (Key)i);
            }

            // add the function keys
            int F1 = (int)Key.F1;
            for (int i = F1; i <= (int)Key.F15; i++)
            {
                keyMap.Add("F" + (i - F1 + 1), (Key)i);
                keyMap.Add("f" + (i - F1 + 1), (Key)i);
            }
        }
        private static void CreateStringToButMap()
        {
            if (butMap != null)
                return;

            butMap = new Hashtable(10);

            butMap.Add("MousePri", MouseButton.PrimaryButton);
            butMap.Add("MousePrimary", MouseButton.PrimaryButton);
            butMap.Add("MouseSec", MouseButton.SecondaryButton);
            butMap.Add("MouseSecondary", MouseButton.SecondaryButton);
            butMap.Add("MouseMid", MouseButton.MiddleButton);
            butMap.Add("MouseMiddle", MouseButton.MiddleButton);
            butMap.Add("MouseWheelUp", MouseButton.WheelUp);
            butMap.Add("MouseWheelDown", MouseButton.WheelDown);
        }
    }
}
