using System;
using System.Drawing;
using System.Collections.Generic;
using SdlDotNet;

using Physics.Entities.Physics;
using Physics.Entities.Particles;

using Physics.Config.Files;

namespace Physics.Entities
{
    // handles run time creation on Entities (reb balls + polygon drawing)
    static class EntityCreator
    {
        // Queues of Entities that are in the EntityManager
        static Queue<Ball> balls = new Queue<Ball>();
        static Queue<EPolygon> polys = new Queue<EPolygon>();

        static EPolygon currentPolygon;
        static Color polyColor = Color.PowderBlue;
        static Stack<Vector2> points = new Stack<Vector2>();

        static bool createWindParticles = false;
        static int numToAdd = 3;
        static Random ran = new Random();

        static Creator_cfg cfg = new Creator_cfg();


        public static void creatingPolygonUpdate()
        {
            if (cfg.addPoint())
            {
                addPoint(new Vector2(Mouse.MousePosition));
                UpdateCurrentPoly();
            }
            else if (cfg.removePoint())
            {
                removePoint();
                UpdateCurrentPoly();
            }

            removePoint();
            addPoint(new Vector2(Mouse.MousePosition));
            UpdateCurrentPoly();

        }
        public static void finalizePolygon()
        {
            if (points.Count > 1)
            {
                removePoint();
                UpdateCurrentPoly();

                polys.Enqueue(currentPolygon);
                EntityManager.Add(currentPolygon);
            }

            currentPolygon = null;
            points.Clear();
        }
        public static void DrawPolygon(Surface target)
        {
            if (currentPolygon != null)
                currentPolygon.Draw(target);
        }

        static void addPoint(Vector2 point)
        {
            points.Push(point);
        }
        static void removePoint()
        {
            if (points.Count != 0)
            {
                points.Pop();
            }
        }
        static void UpdateCurrentPoly()
        {
            if (points.Count != 0)
            {
                currentPolygon = new EPolygon(points.ToArray(), polyColor);
            }
        }

        static void removePoly()
        {
            if (polys.Count == 0)
                return;

            EPolygon removedPoly = polys.Dequeue();
            EntityManager.Remove(removedPoly);
        }
        static void removeAllPolys()
        {
            int numPolys = polys.Count;
            for (int i = 0; i < numPolys; i++)
            {
                EPolygon removedPoly = polys.Dequeue();
                EntityManager.Remove(removedPoly);
            }
        }

        static void addBall(int ballNum)
        {
            Ball newball = new Ball(new Vector2(Mouse.MousePosition));

            if (ballNum == 1)
            {
                newball.Radius = 7;
                newball.color = Color.Red;
                newball.Mass = 0.2F;
            }
            else if (ballNum == 2)
            {
                newball.Radius = 10;
                newball.color = Color.DarkRed;
                newball.Mass = 1F;
            }
            balls.Enqueue(newball);
            EntityManager.Add(newball);
        }
        static void removeBall()
        {
            if (balls.Count == 0)
                return;

            Ball removedBall = balls.Dequeue();
            EntityManager.Remove(removedBall);
        }
        static void removeAllBalls()
        {
            int numBalls = balls.Count;
            for (int i = 0; i < numBalls; i++)
            {
                Ball removedBall = balls.Dequeue();
                EntityManager.Remove(removedBall);
            }
        }

        static void BallInput()
        {
            if (cfg.addBall1())
                addBall(1);
            if (cfg.addBall2())
                addBall(2);

            if (cfg.removeBall())
                removeBall();
            if (cfg.removeAllBalls.wasPressed())
                removeAllBalls();

            if (cfg.printNumBalls.wasPressed())
                Console.WriteLine("Number of created balls: " + balls.Count);
        }
        static void PolyInput()
        {
            if (cfg.removePoly())
                removePoly();
            if (cfg.removeAllPolys.wasPressed())
                removeAllPolys();
        }
        static void WindPInput()
        {
            if (cfg.addWindParticles.wasPressed())
                createWindParticles = !createWindParticles;
        }

        public static void StartEntities()
        {
            Player p1 = new Player(new Vector2(50, 40));
            p1.Radius = 30;
            p1.Mass = 4F;
            p1.keyForce = 0.4F;

            Ball b2 = new Ball(new Vector2(250, 40));
            b2.color = Color.Black;
            b2.Mass = 1F;

            Ball b3 = new Ball(new Vector2(500, 200));
            b3.color = Color.Gray;
            b3.Radius = 50;
            b3.StaticCollisions = true;
            b3.affectedby = new EffectBools(false);

            Vector2[] ps = { 
                new Vector2(50,150), 
                new Vector2(100,200), 
                new Vector2(50,350) };
            EPolygon p = new EPolygon(ps, Color.DarkGreen);

            EntityManager.Add(p1);
            EntityManager.Add(b2);
            EntityManager.Add(b3);
            EntityManager.Add(p);
        }

        public static void HandleInput()
        {
            BallInput();
            PolyInput();
            WindPInput();
        }

        public static void Update(float timeStep)
        {
            if (createWindParticles)
            {
                for (int i = 1; i <= numToAdd; i++)
                {
                    int x = ran.Next(Game.videoSize.X);
                    int y = ran.Next(Game.videoSize.Y);
                    WindParticle p = new WindParticle(new Vector2(x, y), 500);
                    EntityManager.Add(p);
                }
            }
        }
    }
}
