using System;
using System.Drawing;
using SdlDotNet;

namespace Physics.Entities.Physics
{
    // represents a STATIC Polygon that the balls can collide with
    class EPolygon : PhysicsE
    {
        public Color color;
        public Vector2[] screenPoints
        {
            get
            {
                Vector2[] p = new Vector2[pts.Length];
                for (int i = 0; i < pts.Length; i++)
                {
                    p[i] = pts[i].Clone();
                    if ( Rotation != 0 )
                        p[i].Rotate(Rotation);
                    p[i] += Position;
                }

                return p;
            }
        }
        private Vector2[] pts;

        public float maxX;
        public float minX;
        public float maxY;
        public float minY;

        // quick HACK
        public override float RMass
        {
            get { return 1000; }
        }

        // uses the point to approximate a center of mass, then changes arrays to center about 0,0
        public EPolygon(Vector2[] screenPoints, Color color)
        {
            Initalize(screenPoints, color);

            SetCenterOfMass();
        }

        // assumes the points have a center of mass at 0,0 + center
        public EPolygon(Vector2[] points, Color color, Vector2 center)
        {
            Initalize(points, color);

            Position = center.Clone();
        }
        private void Initalize(Vector2[] points, Color color)
        {
            pts = new Vector2[points.Length];

            for (int i = 0; i < points.Length; i++)
                pts[i] = points[i].Clone();

            this.color = color;

            StaticCollisions = true;
        }

        // just averaging the points for now ...
        private void SetCenterOfMass()
        {
            int num = pts.Length;
            Vector2 totP = new Vector2(0, 0);

            for (int i = 0; i < num; i++)
                totP += pts[i];

            this.Position = totP / num;

            for (int i = 0; i < num; i++)
                pts[i] -= Position;
        }

        // used to update the stored Polygon for drawing ...
        private Polygon CreatePoly()
        {
            // WARNING, the sdldotnet creation of a polygon using ArrayList is broken!

            short[] x = new short[pts.Length];
            short[] y = new short[pts.Length];
            Vector2 temp = new Vector2(0, 0);

            for (int i = 0; i < x.Length; i++)
            {
                temp = pts[i].Clone();
                temp.Rotate(Rotation);
                temp += Position;
                x[i] = (short)temp.X;
                y[i] = (short)temp.Y;
            }

            CreateBindingBox();

            return new Polygon(x, y);
        }

        private void CreateBindingBox()
        {
            Vector2[] p = screenPoints;

            if (p.Length == 0)
                return;

            maxX = p[0].X;
            minX = p[0].X;
            maxY = p[0].Y;
            minY = p[0].Y;

            foreach (Vector2 v in p)
            {
                if (maxX < v.X)
                    maxX = v.X;
                if (minX > v.X)
                    minX = v.X;

                if (maxY < v.Y)
                    maxY = v.Y;
                if (minY > v.Y)
                    minY = v.Y;
            }
        }

        public override void Draw(Surface target)
        {
            Vector2[] scrPts = screenPoints;
            if (pts == null) { }
            else if (pts.Length == 1)
            {
                target.DrawPixel((int)scrPts[0].X, (int)scrPts[0].Y, color);
            }
            else if (pts.Length == 2)
            {
                target.DrawLine(new Line((short)scrPts[0].X, (short)scrPts[0].Y, (short)scrPts[1].X, (short)scrPts[1].Y), color, true);
            }
            else
            {
                Polygon Poly = CreatePoly();

                target.DrawFilledPolygon(Poly, color);

                if (antiAliasingOn) // apply some anti-aliasing if true
                    target.DrawPolygon(Poly, color, true);
            }
        }
    }
}

