using System;
using SdlDotNet;

using Physics.Entities;

namespace Physics.States
{
    class Running : GameState
    {
        // The font used for rendering text in the game
        static SdlDotNet.Font gameFont = new SdlDotNet.Font("FreeSans.ttf", 14);

        public override void Input(bool onTop)
        {
            if (onTop)
            {
                if (cfg.pause.wasPressed())
                    Game.AddGameState(new Paused());

                if (cfg.createPoly.wasPressed())
                    Game.AddGameState(new CreatingPolygon());

                // Handles Entity creation input
                EntityCreator.HandleInput();

                // Calls our Manager's Input method which calls all Entity Inputs
                EntityManager.HandleInput();

                // Handles input related to debugging information
                Debugger.HandleInput();
            }
        }

        public override void Update(float timeStep, bool onTop)
        {
            if (onTop)
            {
                EntityCreator.Update(timeStep);
                EntityManager.Update(timeStep);
            }
        }

        public override void Draw(Surface target, bool onTop)
        {
            // draws all objects from EntityManager onto screen
            EntityManager.Draw(target);
        }
    }
}
